from typing import Optional, Callable, Union
from rubigram.types import Update, InlineMessage
from rubigram.method import Method
from rubigram.filters import Filter
from rubigram.state import StateManager
from datetime import datetime
from aiohttp.web import Application, Request, json_response, run_app
import asyncio
import logging


logging.basicConfig(
    format="%(asctime)s | %(levelname)s : %(message)s",
    datefmt="%Y-%m-%d %H:%M:%S",
    level=logging.DEBUG
)

logging.getLogger("asyncio").setLevel(logging.WARNING)
logging.getLogger("aiohttp").setLevel(logging.WARNING)


class Client(Method):
    def __init__(
        self,
        token: str,
        endpoint: Optional[str] = None,
        host: str = "0.0.0.0",
        port: int = 8000
    ):
        self.token = token
        self.endpoint = endpoint
        self.host = host
        self.port = port
        self.offset_id = None
        self.set_endpoint = True
        self.ROUTES = []
        self.MESSAGE_HANDLER = []
        self.INLINE_HANDLER = []
        self.state = StateManager()
        super().__init__(token)
        
    
    def on_message(self, filters: Optional[Filter] = None):
        def decorator(func: Callable) -> Callable:
            async def wrapper(client: Client, update: Update):
                if filters is None or await filters(update):
                    await func(client, update)
                    return True
                return False
            self.MESSAGE_HANDLER.append(wrapper)
            return func
        return decorator

    def on_inline_message(self, filters: Optional[Filter] = None):
        def decorator(func: Callable) -> Callable:
            async def wrapper(client: Client, update: Update):
                if filters is None or await filters(update):
                    await func(client, update)
                    return True
                return False
            self.INLINE_HANDLER.append(wrapper)
            return func
        return decorator

    def on_create_app(self, path: str, method: str = "GET"):
        def decorator(func):
            self.ROUTES.append((path, func, method.upper()))
            return func
        return decorator
    
    async def dispatch(self, update: Union[Update, InlineMessage]):
        handlers = self.MESSAGE_HANDLER if isinstance(update, Update) else self.INLINE_HANDLER
        for handler in handlers:
            matched = await handler(self, update)
            if matched:
                return
            
    async def updater(self, data: dict):
        if "inline_message" in data:
            event = InlineMessage.from_dict(data["inline_message"])
        elif "update" in data:
            event = Update.from_dict(data["update"])
        else: return
        event.client = self
        await self.dispatch(event)
        
    async def set_endpoints(self):
        endpoint_type = ["ReceiveUpdate", "ReceiveInlineMessage"]
        for i in endpoint_type:
            set_endpoint = await self.update_bot_endpoint(f"{self.endpoint}/{i}", i)
            logging.info(f"status set endpoint for {i} : {set_endpoint["status"]}")
        
    async def on_startup(self, app):
        if self.set_endpoint:
            await self.set_endpoints()
        await self.start()

    async def on_cleanup(self, app):
        await self.stop()
        
    def create_request_handler(self):
        async def wrapper(request: Request):
            data = await request.json()
            await self.updater(data)
            return json_response({"status": "OK"})
        return wrapper


    async def update_runner(self):
        try:
            while True:
                get_update = await self.get_update(100, self.offset_id)
                updates = get_update.updates
                if updates:
                    for update in updates:
                        time = update.new_message.time if update.type == "NewMessage" else update.updated_message.time if update.type == "UpdatedMessage" else None
                        time = int(time)
                        now = int(datetime.now().timestamp())
                        if time and (time >= now or time + 2 >= now):
                            update.client = self
                            await self.dispatch(update)
                    self.offset_id = get_update.next_offset_id
        except Exception as error:
            logging.error(error)
        finally:
            await self.stop()
    
    def run(self, set_endpoint = True):
        self.set_endpoint = set_endpoint
        if self.endpoint:
            app = Application()
            app.on_startup(self.on_startup)
            app.on_cleanup(self.on_cleanup)
            
            app.router.add_post("/ReceiveUpdate", self.create_request_handler())
            app.router.add_post("/ReceiveInlineMessage", self.create_request_handler())
            
            for path, func, method in self.ROUTES:
                match method:
                    case "GET":
                        app.router.add_get(path, func)
                    case "POST":
                        app.router.add_post(path, func)
                    case "DELETE":
                        app.router.add_delete(path, func)
                    case "PUT":
                        app.router.add_put(path, func)
                    case "PATCH":
                        app.router.add_patch(path, func)
                        
            run_app(app, host=self.host, port=self.port)

        else:
            try:
                logging.info("Start Bot")
                asyncio.run(self.update_runner())
            except KeyboardInterrupt:
                logging.info("Stop Bot")
            except Exception as error:
                logging.error(error)


from typing import Any

class StateManager:
    def __init__(self):
        self.DATA: dict[str, dict[str, Any]] = {}
        self.STATE: dict[str, str] = {}

        
    async def set_state(self, user_id: str, state: str):
        self.STATE[user_id] = state
    
    
    async def get_state(self, user_id: str):
        return self.STATE.get(user_id)

    
    async def remove_state(self, user_id: str):
        self.STATE.pop(user_id, None)

        
    async def set_data(self, user_id: str, **data):
        if user_id not in self.DATA:
            self.DATA[user_id] = {}
        self.DATA[user_id].update(data)

    
    async def get_data(self, user_id: str, key: str = None):
        data = self.DATA.get(user_id, {})
        return data.get(key) if key else data


    async def remove_data(self, user_id: str, key: str = None):
        if key:
            return self.DATA.get(user_id, {}).pop(key, None)
        return self.DATA.pop(user_id, None)


from .network import Network
from rubigram import enums
from typing import Optional, Union
from rubigram.types import Bot, Chat, Keypad, MessageId, Updates, BotCommand


class Method(Network):
    def __init__(self, token: str):
        super().__init__(token)
        
    def clean_data(self, data: dict):
        return {key: value for key, value in data.items() if value is not None}
        
    async def get_me(self) -> "Bot":
        response = await self.request("getMe", {})
        return Bot.from_dict(response["bot"])

    async def get_chat(self, chat_id: str) -> "Chat":
        response = await self.request("getChat", {"chat_id": chat_id})
        return Chat.from_dict(response["chat"])

    async def get_update(self, limit: int = 1, offset_id: Optional[int] = None) -> "Updates":
        response = await self.request("getUpdates", {"limit": limit, "offset_id": offset_id})
        return Updates.from_dict(response)

    async def get_file(self, file_id: str) -> str:
        response = await self.request("getFile", {"file_id": file_id})
        return response["download_url"]

    async def set_command(self, commands: list[BotCommand]):
        response = await self.request("setCommands", {"bot_commands": [command.asdict() for command in commands]})
        return response

    async def update_bot_endpoint(self, url: str, type: enums.UpdateEndpointType):
        response = await self.request("updateBotEndpoints", {"url": url, "type": type})
        return response

    async def delete_message(self, chat_id: str, message_id: str):
        await self.request("deleteMessage", {"chat_id": chat_id, "message_id": message_id})

    async def remove_chat_keypad(self, chat_id: str):
        await self.request("editChatKeypad", {"chat_id": chat_id, "chat_keypad_type": "Remove"})

    async def edit_chat_keypad(self, chat_id: str, chat_keypad: Keypad):
        await self.request("editChatKeypad", {"chat_id": chat_id, "chat_keypad_type": "New", "chat_keypad": chat_keypad.asdict()})

    async def edit_message_keypad(self, chat_id: str, message_id: str, inline_keypad: Keypad):
        await self.request("editMessageKeypad", {"chat_id": chat_id, "message_id": message_id, "inline_keypad": inline_keypad.asdict()})

    async def edit_message_text(self, chat_id: str, message_id: str, text: str):
        await self.request("editMessageText", {"chat_id": chat_id, "message_id": message_id, "text": text})
        
    async def edit_message(
        self,
        chat_id: str,
        message_id: str,
        text: Optional[str] = None,
        chat_keypad: Optional[Keypad] = None,
        inline_keypad: Optional[Keypad] = None
    ):
        if text:
            await self.edit_message_text(chat_id, message_id, text)
        if chat_keypad:
            await self.edit_chat_keypad(chat_id, chat_keypad)
        if inline_keypad:
            await self.edit_message_keypad(chat_id, message_id, inline_keypad)

    async def forward_message(self, from_chat_id: str, message_id: str, to_chat_id: str, disable_notification: bool = False) -> "MessageId":
        data = {"from_chat_id": from_chat_id, "message_id": message_id, "to_chat_id": to_chat_id, "disable_notification": disable_notification}
        response = await self.request("forwardMessage", data)
        message = MessageId.from_dict(response)
        message.chat_id = to_chat_id
        message.client = self
        return message

    async def send_message(
        self,
        chat_id: str,
        text: str,
        chat_keypad: Keypad = None,
        inline_keypad: Keypad = None,
        chat_keypad_type: Optional[enums.ChatKeypadType] = None,
        disable_notification: bool = None,
        reply_to_message_id=None
    ) -> "MessageId":
        data = {
            "chat_id": chat_id,
            "text": text,
            "chat_keypad": chat_keypad.asdict() if chat_keypad else None,
            "inline_keypad": inline_keypad.asdict() if inline_keypad else None,
            "chat_keypad_type": chat_keypad_type,
            "disable_notification": disable_notification,
            "reply_to_message_id": reply_to_message_id
        }
        response = await self.request("sendMessage", self.clean_data(data))
        message = MessageId.from_dict(response)
        message.chat_id = chat_id
        message.client = self
        return message

    async def send_poll(
        self,
        chat_id: str,
        question: str,
        options: list[str],
        chat_keypad: Keypad = None,
        inline_keypad: Keypad = None,
        chat_keypad_type: Optional[enums.ChatKeypadType] = None,
        disable_notification: bool = False,
        reply_to_message_id: str = None,
    ) -> "MessageId":
        data = {
            "chat_id": chat_id,
            "question": question,
            "options": options,
            "chat_keypad": chat_keypad.asdict() if chat_keypad else None,
            "inline_keypad": inline_keypad.asdict() if inline_keypad else None,
            "disable_notification": disable_notification,
            "reply_to_message_id": reply_to_message_id,
            "chat_keypad_type": chat_keypad_type
        }
        response = await self.request("sendPoll", self.clean_data(data))
        message = MessageId.from_dict(response)
        message.chat_id = chat_id
        message.client = self
        return message

    async def send_location(
        self,
        chat_id: str,
        latitude: str,
        longitude: str,
        chat_keypad: Keypad = None,
        inline_keypad: Keypad = None,
        chat_keypad_type: Optional[enums.ChatKeypadType] = None,
        disable_notification: bool = False,
        reply_to_message_id: str = None,
    ) -> "MessageId":
        data = {
            "chat_id": chat_id,
            "latitude": latitude,
            "longitude": longitude,
            "chat_keypad": chat_keypad.asdict() if chat_keypad else None,
            "inline_keypad": inline_keypad.asdict() if inline_keypad else None,
            "disable_notification": disable_notification,
            "reply_to_message_id": reply_to_message_id,
            "chat_keypad_type": chat_keypad_type
        }
        response = await self.request("sendLocation", self.clean_data(data))
        message = MessageId.from_dict(response)
        message.chat_id = chat_id
        message.client = self
        return message

    async def send_contact(
        self,
        chat_id: str,
        first_name: str,
        last_name: str,
        phone_number: str,
        chat_keypad: Keypad = None,
        inline_keypad: Keypad = None,
        chat_keypad_type: Optional[enums.ChatKeypadType] = None,
        disable_notification: bool = False,
        reply_to_message_id: str = None,
    ) -> "MessageId":
        data = {
            "chat_id": chat_id,
            "first_name": first_name,
            "last_name": last_name,
            "phone_number": phone_number,
            "chat_keypad": chat_keypad.asdict() if chat_keypad else None,
            "inline_keypad": inline_keypad.asdict() if inline_keypad else None,
            "disable_notification": disable_notification,
            "reply_to_message_id": reply_to_message_id,
            "chat_keypad_type": chat_keypad_type
        }
        response = await self.request("sendContact", self.clean_data(data))
        message = MessageId.from_dict(response)
        message.chat_id = chat_id
        message.client = self
        return message

    async def send_sticker(
        self,
        chat_id: str,
        sticker_id: str,
        chat_keypad: Keypad = None,
        inline_keypad: Keypad = None,
        chat_keypad_type: Optional[enums.ChatKeypadType] = None,
        disable_notification: bool = False,
        reply_to_message_id: str = None,
    ) -> "MessageId":
        data = {
            "chat_id": chat_id,
            "sticker_id": sticker_id,
            "chat_keypad": chat_keypad.asdict() if chat_keypad else None,
            "inline_keypad": inline_keypad.asdict() if inline_keypad else None,
            "disable_notification": disable_notification,
            "reply_to_message_id": reply_to_message_id,
            "chat_keypad_type": chat_keypad_type
        }
        response = await self.request("sendSticker", self.clean_data(data))
        message = MessageId.from_dict(response)
        message.chat_id = chat_id
        message.client = self
        return message

    async def request_send_file(self, type: str):
        response = await self.request("requestSendFile", {"type": type})
        return response["upload_url"]

    async def upload_file(self, file: Union[str, bytes], name: Optional[str] = None, type: str = "File"):
        upload_url = await self.request_send_file(type)
        response = await self.requestUpload(upload_url, file, name)
        return response

    async def download_file(self, file_id: str, filename: Optional[str] = None):
        download_url = await self.get_file(file_id)
        response = await self.requestDownload(download_url, filename)
        return response

    async def send_file(
        self,
        chat_id: str,
        file: Union[str, bytes],
        caption: Optional[str] = None,
        file_name: Optional[str] = None,
        type: enums.FileType = enums.FileType.File,
        chat_keypad: Keypad = None,
        inline_keypad: Keypad = None,
        chat_keypad_type: Optional[enums.ChatKeypadType] = None,
        disable_notification: bool = False,
        reply_to_message_id: Optional[str] = None,
    ) -> "MessageId":
        file_id = await self.upload_file(file, file_name, type)
        
        data = {
            "chat_id": chat_id,
            "file_id": file_id,
            "text": caption,
            "chat_keypad": chat_keypad.asdict() if chat_keypad else None,
            "inline_keypad": inline_keypad.asdict() if inline_keypad else None,
            "disable_notification": disable_notification,
            "reply_to_message_id": reply_to_message_id,
            "chat_keypad_type": chat_keypad_type,
        }
        response = await self.request("sendFile", self.clean_data(data))
        message = MessageId.from_dict(response)
        message.chat_id = chat_id
        message.file_id = file_id
        message.client = self
        return message



    async def send_document(self, chat_id: str, document: Union[str, bytes], caption: Optional[str] = None, file_name: Optional[str] = None, **kwargs):
        return await self.send_file(chat_id, document, caption, file_name, "File", **kwargs)

    async def send_photo(self, chat_id: str, photo: Union[str, bytes], caption: Optional[str] = None, file_name: Optional[str] = None, **kwargs):
        return await self.send_file(chat_id, photo, caption, file_name, "Image", **kwargs)

    async def send_video(self, chat_id: str, video: Union[str, bytes], caption: Optional[str] = None, file_name: Optional[str] = None, **kwargs):
        return await self.send_file(chat_id, video, caption, file_name, "Video", **kwargs)

    async def send_gif(self, chat_id: str, gif: Union[str, bytes], caption: Optional[str] = None, file_name: Optional[str] = None, **kwargs):
        return await self.send_file(chat_id, gif, caption, file_name, "Gif", **kwargs)

    async def send_music(self, chat_id: str, music: Union[str, bytes], caption: Optional[str] = None, file_name: Optional[str] = None, **kwargs):
        return await self.send_file(chat_id, music, caption, file_name, "Music", **kwargs)

    async def send_voice(self, chat_id: str, voice: Union[str, bytes], caption: Optional[str] = None, file_name: Optional[str] = None, **kwargs):
        return await self.send_file(chat_id, voice, caption, file_name, "Voice", **kwargs)
    
    # async def get_messages(self, chat_id: str, message_ids: Optional[list[str]] = None):
    #     return await self.request("getMessages", {"chat_id": chat_id, "message_ids": message_ids})
        
    # async def get_bot_command(self):
    #     return await self.request("getBotCommands", {})
    
    # async def send_chat_action(self, chat_id: str, action: enums.ChatAction):
    #     return await self.request("sendChatAction", {"chat_id": chat_id, "action": action})
    
    # async def set_username(self, username: str):
    #     return await self.request("setUsername", {"username": username})
    
    async def get_bot_name(self):
        bot = await self.get_me()
        return bot.bot_title

    async def get_bot_id(self):
        bot = await self.get_me()
        return bot.bot_id
    
    async def get_bot_username(self):
        bot = await self.get_me()
        return bot.username
    
    async def get_bot_share_url(self):
        bot = await self.get_me()
        return bot.share_url
    
    async def get_bot_descriptionl(self):
        bot = await self.get_me()
        return bot.description
    
    async def get_bot_start_messagel(self):
        bot = await self.get_me()
        return bot.start_message


from aiohttp import ClientSession
from typing import Literal, Optional, Union, Any
from random import randint


class Rubino:
    def __init__(self, auth: str):
        self.auth = auth
        self.api = f"https://rubino{randint(1, 30)}.iranlms.ir"
        self.client = {
            "app_name": "Main",
            "app_version": "3.0.2",
            "lang_code": "fa",
            "package": "app.rbmain.a",
            "platform": "Android"
        }

    
    async def reauest(self, method: str, data: dict[str, Any]):
        json = {
            "api_version": "0",
            "auth": self.auth,
            "client": self.client,
            "data": data,
            "method": method
        }
        async with ClientSession() as session:
            async with session.post(self.api, json=json) as response:
                response.raise_for_status()
                return await response.json()

    
    async def get_post_by_share_link(self, post_link: str):
        return await self.reauest("getPostByShareLink", {"share_string": post_link.split("/")[-1]})
    
    
    async def add_post_view_count(self, post_id: str, post_profile_id: str):
        data = {"post_id": post_id, "post_profile_id": post_profile_id}
        return await self.reauest("addPostViewCount", data)
    
    
    async def add_view_story(self, story_profile_id: str, story_ids: Union[str, list[str]], profile_id: Optional[str] = None):
        story_ids = story_ids if isinstance(story_ids, list) else [story_ids]
        data = {"story_profile_id": story_profile_id, "story_ids": story_ids, "profile_id": profile_id}
        return await self.reauest("addViewStory", data)
    
    
    async def is_exist_username(self, username: str):
        username = username.replace("@", "")
        return await self.reauest("isExistUsername", {"username": username})
    
    
    async def create_page(self, username: str, name: str, bio: Optional[str] = None):
        return await self.reauest("createPage", {"username": username, "name": name, "bio": bio})
    
    
    async def add_comment(self, content: str, post_id: str, post_profile_id: str, profile_id: Optional[str] = None):
        rnd = randint(100000, 999999999)
        data = {"content": content, "post_id": post_id, "post_profile_id": post_profile_id, "profile_id": profile_id, "rnd": rnd}
        return await self.reauest("addComment", data)
    
    
    async def request_follow(self, followee_id: str, f_type: Literal["Follow", "Unfollow"] = "Follow", profile_id: Optional[str] = None):
        data = {"f_type": f_type, "followee_id": followee_id, "profile_id": profile_id}
        return self.reauest("requestFollow", data)
    
    
    async def set_block_profile(self, block_id: str, action: Literal["Block", "Unblock"] = "Block", profile_id: Optional[str] = None):
        data = {"block_id": block_id, "action": action, "profile_id": profile_id}
        return self.reauest("setBlockProfile", data)
    
    
    async def get_comments(self, post_id: str, post_profile_id: str, limit: Optional[int] = 100, profile_id: Optional[str] = None):
        data = {
            "post_id": post_id,
            "post_profile_id": post_profile_id,
            "limit": limit,
            "profile_id": profile_id,
            "equal": False,
            "sort": "FromMax"
        }
        return await self.reauest("getComments", data)


from enum import Enum


class ChatType(str, Enum):
    User = "User"
    Bot = "Bot"
    Group = "Group"
    Channel = "Channel"


class ForwardedFrom(str, Enum):
    User = "User"
    Bot = "Bot"
    Channel = "Channel"


class PaymentStatus(str, Enum):
    Paid = "Paid"
    NotPaid = "NotPaid"


class PollStatus(str, Enum):
    Open = "Open"
    Closed = "Closed"


class LiveLocationStatus(str, Enum):
    Stopped = "Stopped"
    Live = "Live"


class ButtonSelectionType(str, Enum):
    TextOnly = "TextOnly"
    TextImgThu = "TextImgThu"
    TextImgBig = "TextImgBig"


class ButtonSelectionSearch(str, Enum):
    Local = "Local"
    Api = "Api"


class ButtonSelectionGet(str, Enum):
    Local = "Local"
    Api = "Api"


class ButtonCalendarType(str, Enum):
    DatePersian = "DatePersian"
    DateGregorian = "DateGregorian"


class ButtonTextboxTypeKeypad(str, Enum):
    String = "String"
    Number = "Number"


class ButtonTextboxTypeLine(str, Enum):
    SingleLine = "SingleLine"
    MultiLine = "MultiLine"


class ButtonLocationType(str, Enum):
    Picker = "Picker"
    View = "View"


class ButtonLinkType(str, Enum):
    joinchannel = "joinchannel"
    url = "url"

class MessageSender(str, Enum):
    User = "User"
    Bot = "Bot"


class UpdateType(str, Enum):
    UpdatedMessage = "UpdatedMessage"
    NewMessage = "NewMessage"
    RemovedMessage = "RemovedMessage"
    StartedBot = "StartedBot"
    StoppedBot = "StoppedBot"
    UpdatedPayment = "UpdatedPayment"


class ChatKeypadType(str, Enum):
    New = "New"
    Remove = "Remove"


class UpdateEndpointType(str, Enum):
    ReceiveUpdate = "ReceiveUpdate"
    ReceiveInlineMessage = "ReceiveInlineMessage"
    ReceiveQuery = "ReceiveQuery"
    GetSelectionItem = "GetSelectionItem"
    SearchSelectionItems = "SearchSelectionItems"
    

class ButtonType(str, Enum):
    Simple = "Simple"
    Selection = "Selection"
    Calendar = "Calendar"
    NumberPicker = "NumberPicker"
    StringPicker = "StringPicker"
    Location = "Location"
    Payment = "Payment"
    CameraImage = "CameraImage"
    CameraVideo = "CameraVideo"
    GalleryImage = "GalleryImage"
    GalleryVideo = "GalleryVideo"
    File = "File"
    Audio = "Audio"
    RecordAudio = "RecordAudio"
    MyPhoneNumber = "MyPhoneNumber"
    MyLocation = "MyLocation"
    Textbox = "Textbox"
    Link = "Link"
    AskMyPhoneNumber = "AskMyPhoneNumber"
    AskLocation = "AskLocation"
    Barcode = "Barcode"
    

class FileType(str, Enum):
    File = "File"
    Image = "Image"
    Video = "Video"
    Gif = "Gif"
    Music = "Music"
    Voice = "Voice"
    

class ChatAction(str, Enum):
    Typing = "Typing"
    Uploading = "Uploading"
    Recording = "Recording"
    
    


from typing import Optional, Union, Any
from pydantic import BaseModel
from rubigram import enums


class DataManager(BaseModel):
    class Config:
        use_enum_values = True
        arbitrary_types_allowed = True

    @classmethod
    def from_dict(cls, data: dict):
        return cls.model_validate(data or {})

    def asdict(self, exclude_none: bool = True) -> dict:
        return self.model_dump(exclude_none=exclude_none, exclude={"client"})

    def asjson(self, exclude_none: bool = True) -> str:
        return self.model_dump_json(indent=4, exclude_none=exclude_none, exclude={"client"})


class Chat(DataManager):
    chat_id: Optional[str] = None
    chat_type: Optional[enums.ChatType] = None
    user_id: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None
    title: Optional[str] = None
    username: Optional[str] = None

    def __init__(
        self,
        chat_id: Optional[str] = None,
        chat_type: Optional[enums.ChatType] = None,
        user_id: Optional[str] = None,
        first_name: Optional[str] = None,
        last_name: Optional[str] = None,
        title: Optional[str] = None,
        username: Optional[str] = None,
        **kwargs
    ):
        super().__init__(
            chat_id=chat_id,
            chat_type=chat_type,
            user_id=user_id,
            first_name=first_name,
            last_name=last_name,
            title=title,
            username=username,
            **kwargs
        )


class File(DataManager):
    file_id: Optional[str] = None
    file_name: Optional[str] = None
    size: Optional[str] = None

    def __init__(
        self,
        file_id: Optional[str] = None,
        file_name: Optional[str] = None,
        size: Optional[str] = None,
        **kwargs
    ):
        super().__init__(
            file_id=file_id,
            file_name=file_name,
            size=size,
            **kwargs
        )


class ForwardedFrom(DataManager):
    type_from: Optional[enums.ForwardedFrom] = None
    message_id: Optional[str] = None
    from_chat_id: Optional[str] = None
    from_sender_id: Optional[str] = None

    def __init__(
        self,
        type_from: Optional[enums.ForwardedFrom] = None,
        message_id: Optional[str] = None,
        from_chat_id: Optional[str] = None,
        from_sender_id: Optional[str] = None,
        **kwargs
    ):
        super().__init__(
            type_from=type_from,
            message_id=message_id,
            from_chat_id=from_chat_id,
            from_sender_id=from_sender_id,
            **kwargs
        )


class PaymentStatus(DataManager):
    payment_id: Optional[str] = None
    status: Optional[enums.PaymentStatus] = None

    def __init__(
        self,
        payment_id: Optional[str] = None,
        status: Optional[enums.PaymentStatus] = None,
        **kwargs
    ):
        super().__init__(
            payment_id=payment_id,
            status=status,
            **kwargs
        )


class MessageTextUpdate(DataManager):
    message_id: Optional[str] = None
    text: Optional[str] = None

    def __init__(
        self,
        message_id: Optional[str] = None,
        text: Optional[str] = None,
        **kwargs
    ):
        super().__init__(
            message_id=message_id,
            text=text,
            **kwargs
        )


class Bot(DataManager):
    bot_id: Optional[str] = None
    bot_title: Optional[str] = None
    avatar: Optional[File] = None
    description: Optional[str] = None
    username: Optional[str] = None
    start_message: Optional[str] = None
    share_url: Optional[str] = None

    def __init__(
        self,
        bot_id: Optional[str] = None,
        bot_title: Optional[str] = None,
        avatar: Optional[File] = None,
        description: Optional[str] = None,
        username: Optional[str] = None,
        start_message: Optional[str] = None,
        share_url: Optional[str] = None,
        **kwargs
    ):
        super().__init__(
            bot_id=bot_id,
            bot_title=bot_title,
            avatar=avatar,
            description=description,
            username=username,
            start_message=start_message,
            share_url=share_url,
            **kwargs
        )


class BotCommand(DataManager):
    command: Optional[str] = None
    description: Optional[str] = None

    def __init__(
        self,
        command: Optional[str] = None,
        description: Optional[str] = None,
        **kwargs
    ):
        super().__init__(
            command=command,
            description=description,
            **kwargs
        )


class Sticker(DataManager):
    sticker_id: Optional[str] = None
    file: Optional[File] = None
    emoji_character: Optional[str] = None

    def __init__(
        self,
        sticker_id: Optional[str] = None,
        file: Optional[File] = None,
        emoji_character: Optional[str] = None,
        **kwargs
    ):
        super().__init__(
            sticker_id=sticker_id,
            file=file,
            emoji_character=emoji_character,
            **kwargs
        )


class ContactMessage(DataManager):
    phone_number: Optional[str] = None
    first_name: Optional[str] = None
    last_name: Optional[str] = None

    def __init__(
        self,
        phone_number: Optional[str] = None,
        first_name: Optional[str] = None,
        last_name: Optional[str] = None,
        **kwargs
    ):
        super().__init__(
            phone_number=phone_number,
            first_name=first_name,
            last_name=last_name,
            **kwargs
        )


class PollStatus(DataManager):
    state: Optional[enums.PollStatus] = None
    selection_index: Optional[int] = None
    percent_vote_options: Optional[list[int]] = None
    total_vote: Optional[int] = None
    show_total_votes: Optional[bool] = None

    def __init__(
        self,
        state: Optional[enums.PollStatus] = None,
        selection_index: Optional[int] = None,
        percent_vote_options: Optional[list[int]] = None,
        total_vote: Optional[int] = None,
        show_total_votes: Optional[bool] = None,
        **kwargs
    ):
        super().__init__(
            state=state,
            selection_index=selection_index,
            percent_vote_options=percent_vote_options,
            total_vote=total_vote,
            show_total_votes=show_total_votes,
            **kwargs
        )


class Poll(DataManager):
    question: Optional[str] = None
    options: Optional[list[str]] = None
    poll_status: Optional[PollStatus] = None

    def __init__(
        self,
        question: Optional[str] = None,
        options: Optional[list[str]] = None,
        poll_status: Optional[PollStatus] = None,
        **kwargs
    ):
        super().__init__(
            question=question,
            options=options,
            poll_status=poll_status,
            **kwargs
        )


class Location(DataManager):
    longitude: Optional[str] = None
    latitude: Optional[str] = None

    def __init__(
        self,
        longitude: Optional[str] = None,
        latitude: Optional[str] = None,
        **kwargs
    ):
        super().__init__(
            longitude=longitude,
            latitude=latitude,
            **kwargs
        )


class LiveLocation(DataManager):
    start_time: Optional[str] = None
    live_period: Optional[int] = None
    current_location: Optional[Location] = None
    user_id: Optional[str] = None
    status: Optional[enums.LiveLocationStatus] = None
    last_update_time: Optional[str] = None

    def __init__(
        self,
        start_time: Optional[str] = None,
        live_period: Optional[int] = None,
        current_location: Optional[Location] = None,
        user_id: Optional[str] = None,
        status: Optional[enums.LiveLocationStatus] = None,
        last_update_time: Optional[str] = None,
        **kwargs
    ):
        super().__init__(
            start_time=start_time,
            live_period=live_period,
            current_location=current_location,
            user_id=user_id,
            status=status,
            last_update_time=last_update_time,
            **kwargs
        )


class ButtonSelectionItem(DataManager):
    text: Optional[str] = None
    image_url: Optional[str] = None
    type: Optional[enums.ButtonSelectionType] = None

    def __init__(
        self,
        text: Optional[str] = None,
        image_url: Optional[str] = None,
        type: Optional[enums.ButtonSelectionType] = None,
        **kwargs
    ):
        super().__init__(
            text=text,
            image_url=image_url,
            type=type,
            **kwargs
        )


class ButtonSelection(DataManager):
    selection_id: Optional[str] = None
    search_type: Optional[str] = None
    get_type: Optional[str] = None
    items: Optional[list[ButtonSelectionItem]] = None
    is_multi_selection: Optional[bool] = None
    columns_count: Optional[str] = None
    title: Optional[str] = None

    def __init__(
        self,
        selection_id: Optional[str] = None,
        search_type: Optional[str] = None,
        get_type: Optional[str] = None,
        items: Optional[list[ButtonSelectionItem]] = None,
        is_multi_selection: Optional[bool] = None,
        columns_count: Optional[str] = None,
        title: Optional[str] = None,
        **kwargs
    ):
        super().__init__(
            selection_id=selection_id,
            search_type=search_type,
            get_type=get_type,
            items=items,
            is_multi_selection=is_multi_selection,
            columns_count=columns_count,
            title=title,
            **kwargs
        )


class ButtonCalendar(DataManager):
    default_value: Optional[str] = None
    type: Optional[enums.ButtonCalendarType] = None
    min_year: Optional[str] = None
    max_year: Optional[str] = None
    title: Optional[str] = None

    def __init__(
        self,
        default_value: Optional[str] = None,
        type: Optional[enums.ButtonCalendarType] = None,
        min_year: Optional[str] = None,
        max_year: Optional[str] = None,
        title: Optional[str] = None,
        **kwargs
    ):
        super().__init__(
            default_value=default_value,
            type=type,
            min_year=min_year,
            max_year=max_year,
            title=title,
            **kwargs
        )


class ButtonNumberPicker(DataManager):
    min_value: Optional[str] = None
    max_value: Optional[str] = None
    default_value: Optional[str] = None
    title: Optional[str] = None

    def __init__(
        self,
        min_value: Optional[str] = None,
        max_value: Optional[str] = None,
        default_value: Optional[str] = None,
        title: Optional[str] = None,
        **kwargs
    ):
        super().__init__(
            min_value=min_value,
            max_value=max_value,
            default_value=default_value,
            title=title,
            **kwargs
        )


class ButtonStringPicker(DataManager):
    items: Optional[list[str]] = None
    default_value: Optional[str] = None
    title: Optional[str] = None

    def __init__(
        self,
        items: Optional[list[str]] = None,
        default_value: Optional[str] = None,
        title: Optional[str] = None,
        **kwargs
    ):
        super().__init__(
            items=items,
            default_value=default_value,
            title=title,
            **kwargs
        )


class ButtonTextbox(DataManager):
    type_line: Optional[enums.ButtonTextboxTypeLine] = None
    type_keypad: Optional[enums.ButtonTextboxTypeKeypad] = None
    place_holder: Optional[str] = None
    title: Optional[str] = None
    default_value: Optional[str] = None

    def __init__(
        self,
        type_line: Optional[enums.ButtonTextboxTypeLine] = None,
        type_keypad: Optional[enums.ButtonTextboxTypeKeypad] = None,
        place_holder: Optional[str] = None,
        title: Optional[str] = None,
        default_value: Optional[str] = None,
        **kwargs
    ):
        super().__init__(
            type_line=type_line,
            type_keypad=type_keypad,
            place_holder=place_holder,
            title=title,
            default_value=default_value,
            **kwargs
        )


class ButtonLocation(DataManager):
    default_pointer_location: Optional[Location] = None
    default_map_location: Optional[Location] = None
    type: Optional[enums.ButtonLocationType] = None
    title: Optional[str] = None
    location_image_url: Optional[str] = None

    def __init__(
        self,
        default_pointer_location: Optional[Location] = None,
        default_map_location: Optional[Location] = None,
        type: Optional[enums.ButtonLocationType] = None,
        title: Optional[str] = None,
        location_image_url: Optional[str] = None,
        **kwargs
    ):
        super().__init__(
            default_pointer_location=default_pointer_location,
            default_map_location=default_map_location,
            type=type,
            title=title,
            location_image_url=location_image_url,
            **kwargs
        )


class OpenChatData(DataManager):
    object_guid: Optional[str] = None
    object_type: Optional[enums.ChatType] = None

    def __init__(
        self,
        object_guid: Optional[str] = None,
        object_type: Optional[enums.ChatType] = None,
        **kwargs
    ):
        super().__init__(
            object_guid=object_guid,
            object_type=object_type,
            **kwargs
        )


class JoinChannelData(DataManager):
    username: Optional[str] = None
    ask_join: bool = False

    def __init__(
        self,
        username: Optional[str] = None,
        ask_join: bool = False,
        **kwargs
    ):
        super().__init__(
            username=username,
            ask_join=ask_join,
            **kwargs
        )


class ButtonLink(DataManager):
    type: Optional[enums.ButtonLinkType] = None
    link_url: Optional[str] = None
    joinchannel_data: Optional[JoinChannelData] = None
    open_chat_data: Optional[OpenChatData] = None

    def __init__(
        self,
        type: Optional[enums.ButtonLinkType] = None,
        link_url: Optional[str] = None,
        joinchannel_data: Optional[JoinChannelData] = None,
        open_chat_data: Optional[OpenChatData] = None,
        **kwargs
    ):
        super().__init__(
            type=type,
            link_url=link_url,
            joinchannel_data=joinchannel_data,
            open_chat_data=open_chat_data,
            **kwargs
        )


class AuxData(DataManager):
    start_id: Optional[str] = None
    button_id: Optional[str] = None

    def __init__(
        self,
        start_id: Optional[str] = None,
        button_id: Optional[str] = None,
        **kwargs
    ):
        super().__init__(
            start_id=start_id,
            button_id=button_id,
            **kwargs
        )


class Button(DataManager):
    id: Optional[str] = None
    button_text: Optional[str] = None
    type: enums.ButtonType = enums.ButtonType.Simple
    button_selection: Optional[ButtonSelection] = None
    button_calendar: Optional[ButtonCalendar] = None
    button_number_picker: Optional[ButtonNumberPicker] = None
    button_string_picker: Optional[ButtonStringPicker] = None
    button_location: Optional[ButtonLocation] = None
    button_textbox: Optional[ButtonTextbox] = None
    button_link: Optional[ButtonLink] = None

    def __init__(
        self,
        id: Optional[str] = None,
        button_text: Optional[str] = None,
        type: enums.ButtonType = enums.ButtonType.Simple,
        button_selection: Optional[ButtonSelection] = None,
        button_calendar: Optional[ButtonCalendar] = None,
        button_number_picker: Optional[ButtonNumberPicker] = None,
        button_string_picker: Optional[ButtonStringPicker] = None,
        button_location: Optional[ButtonLocation] = None,
        button_textbox: Optional[ButtonTextbox] = None,
        button_link: Optional[ButtonLink] = None,
        **kwargs
    ):
        super().__init__(
            id=id,
            button_text=button_text,
            type=type,
            button_selection=button_selection,
            button_calendar=button_calendar,
            button_number_picker=button_number_picker,
            button_string_picker=button_string_picker,
            button_location=button_location,
            button_textbox=button_textbox,
            button_link=button_link,
            **kwargs
        )


class KeypadRow(DataManager):
    buttons: list[Button]

    def __init__(
        self,
        buttons: list[Button],
        **kwargs
    ):
        super().__init__(
            buttons=buttons,
            **kwargs
        )


class Keypad(DataManager):
    rows: list[KeypadRow]
    resize_keyboard: bool = True
    on_time_keyboard: bool = False

    def __init__(
        self,
        rows: list[KeypadRow],
        resize_keyboard: bool = True,
        on_time_keyboard: bool = False,
        **kwargs
    ):
        super().__init__(
            rows=rows,
            resize_keyboard=resize_keyboard,
            on_time_keyboard=on_time_keyboard,
            **kwargs
        )


class MessageKeypadUpdate(DataManager):
    message_id: Optional[str] = None
    inline_keypad: Optional[Keypad] = None

    def __init__(
        self,
        message_id: Optional[str] = None,
        inline_keypad: Optional[Keypad] = None,
        **kwargs
    ):
        super().__init__(
            message_id=message_id,
            inline_keypad=inline_keypad,
            **kwargs
        )


class Message(DataManager):
    message_id: Optional[str] = None
    text: Optional[str] = None
    time: Optional[str] = None
    is_edited: Optional[bool] = None
    sender_type: Optional[enums.MessageSender] = None
    sender_id: Optional[str] = None
    aux_data: Optional[AuxData] = None
    file: Optional[File] = None
    reply_to_message_id: Optional[str] = None
    forwarded_from: Optional[ForwardedFrom] = None
    forwarded_no_link: Optional[str] = None
    location: Optional[Location] = None
    sticker: Optional[Sticker] = None
    contact_message: Optional[ContactMessage] = None
    poll: Optional[Poll] = None
    live_location: Optional[LiveLocation] = None

    def __init__(
        self,
        message_id: Optional[str] = None,
        text: Optional[str] = None,
        time: Optional[str] = None,
        is_edited: Optional[bool] = None,
        sender_type: Optional[enums.MessageSender] = None,
        sender_id: Optional[str] = None,
        aux_data: Optional[AuxData] = None,
        file: Optional[File] = None,
        reply_to_message_id: Optional[str] = None,
        forwarded_from: Optional[ForwardedFrom] = None,
        forwarded_no_link: Optional[str] = None,
        location: Optional[Location] = None,
        sticker: Optional[Sticker] = None,
        contact_message: Optional[ContactMessage] = None,
        poll: Optional[Poll] = None,
        live_location: Optional[LiveLocation] = None,
        **kwargs
    ):
        super().__init__(
            message_id=message_id,
            text=text,
            time=time,
            is_edited=is_edited,
            sender_type=sender_type,
            sender_id=sender_id,
            aux_data=aux_data,
            file=file,
            reply_to_message_id=reply_to_message_id,
            forwarded_from=forwarded_from,
            forwarded_no_link=forwarded_no_link,
            location=location,
            sticker=sticker,
            contact_message=contact_message,
            poll=poll,
            live_location=live_location,
            **kwargs
        )


class MessageId(DataManager):
    message_id: Optional[str] = None
    file_id: Optional[str] = None
    chat_id: Optional[str] = None
    client: Optional[Any] = None

    def __init__(
        self,
        message_id: Optional[str] = None,
        file_id: Optional[str] = None,
        chat_id: Optional[str] = None,
        client: Optional[Any] = None,
        **kwargs
    ):
        super().__init__(
            message_id=message_id,
            file_id=file_id,
            chat_id=chat_id,
            client=client,
            **kwargs
        )

    async def delete(self):
        return await self.client.delete_message(self.chat_id, self.message_id)
    
    async def edit(self, text: Optional[str] = None, inline: Optional[Keypad] = None, keypad: Optional[Keypad] = None):
        if text:
            await self.edit_text(text)
        if inline:
            await self.edit_inline(inline)
        if keypad:
            await self.edit_keypad(keypad)

    async def edit_text(self, text: str):
        return await self.client.edit_message_text(self.chat_id, self.message_id, text)

    async def edit_inline(self, inline: Keypad):
        return await self.client.edit_message_keypad(self.chat_id, self.message_id, inline)
    
    async def edit_keypad(self, keypad: Keypad):
        return await self.client.edit_chat_keypad(self.chat_id, keypad)

    async def forward(self, chat_id: str):
        return await self.client.forward_message(self.chat_id, self.message_id, chat_id)


class Update(DataManager):
    client: Optional[Any] = None
    type: Optional[enums.UpdateType] = None
    chat_id: Optional[str] = None
    removed_message_id: Optional[str] = None
    new_message: Optional[Message] = None
    updated_message: Optional[Message] = None
    updated_payment: Optional[PaymentStatus] = None

    def __init__(
        self,
        type: Optional[enums.UpdateType] = None,
        chat_id: Optional[str] = None,
        removed_message_id: Optional[str] = None,
        new_message: Optional[Message] = None,
        updated_message: Optional[Message] = None,
        updated_payment: Optional[PaymentStatus] = None,
        **kwargs
    ):
        super().__init__(
            type=type,
            chat_id=chat_id,
            removed_message_id=removed_message_id,
            new_message=new_message,
            updated_message=updated_message,
            updated_payment=updated_payment,
            **kwargs
        )

    async def download(self, file_name: str):
        return await self.client.download_file(self.new_message.file.file_id, file_name)

    async def forward(self, chat_id: str):
        return await self.client.forward_message(self.chat_id, self.new_message.message_id, chat_id)

    async def reply(
        self,
        text: str,
        chat_keypad: Keypad = None,
        inline_keypad: Keypad = None,
        chat_keypad_type: Optional[enums.ChatKeypadType] = None,
        disable_notification: bool = None,
    ) -> "MessageId":
        return await self.client.send_message(
            self.chat_id,
            text,
            chat_keypad,
            inline_keypad,
            chat_keypad_type,
            disable_notification,
            self.new_message.message_id,
        )

    async def reply_poll(
        self,
        question: str,
        options: list[str],
        chat_keypad: Keypad = None,
        inline_keypad: Keypad = None,
        chat_keypad_type: Optional[enums.ChatKeypadType] = None,
        disable_notification: bool = False,
    ) -> "MessageId":
        return await self.client.send_poll(
            self.chat_id,
            question,
            options,
            chat_keypad,
            inline_keypad,
            chat_keypad_type,
            disable_notification,
            self.new_message.message_id,
        )

    async def reply_location(
        self,
        latitude: str,
        longitude: str,
        chat_keypad: Keypad = None,
        inline_keypad: Keypad = None,
        chat_keypad_type: Optional[enums.ChatKeypadType] = None,
        disable_notification: bool = False,
    ) -> "MessageId":
        return await self.client.send_location(
            self.chat_id,
            latitude,
            longitude,
            chat_keypad,
            inline_keypad,
            chat_keypad_type,
            disable_notification,
            self.new_message.message_id,
        )

    async def reply_contact(
        self,
        first_name: str,
        last_name: str,
        phone_number: str,
        chat_keypad: Keypad = None,
        inline_keypad: Keypad = None,
        chat_keypad_type: Optional[enums.ChatKeypadType] = None,
        disable_notification: bool = False,
    ) -> "MessageId":
        return await self.client.send_location(
            self.chat_id,
            first_name,
            last_name,
            phone_number,
            chat_keypad,
            inline_keypad,
            chat_keypad_type,
            disable_notification,
            self.new_message.message_id,
        )

    async def reply_sticker(
        self,
        sticker_id: str,
        chat_keypad: Keypad = None,
        inline_keypad: Keypad = None,
        chat_keypad_type: Optional[enums.ChatKeypadType] = None,
        disable_notification: bool = False,
    ) -> "MessageId":
        return await self.client.send_message(
            self.chat_id,
            sticker_id,
            chat_keypad,
            inline_keypad,
            chat_keypad_type,
            disable_notification,
            self.new_message.message_id,
        )

    async def reply_file(
        self,
        file: Union[str, bytes],
        caption: Optional[str] = None,
        file_name: Optional[str] = None,
        type: enums.FileType = enums.FileType.File,
        chat_keypad: Keypad = None,
        inline_keypad: Keypad = None,
        chat_keypad_type: Optional[enums.ChatKeypadType] = None,
        disable_notification: bool = False,
    ) -> "MessageId":
        return await self.client.send_file(
            self.chat_id,
            file,
            caption,
            file_name,
            type,
            chat_keypad,
            inline_keypad,
            chat_keypad_type,
            disable_notification,
            self.new_message.message_id,
        )

    async def reply_document(self, document: Union[str, bytes], caption: Optional[str] = None, file_name: Optional[str] = None, **kwargs) -> "MessageId":
        return await self.reply_file(document, caption, file_name, "File", **kwargs)

    async def reply_photo(self, photo: Union[str, bytes], caption: Optional[str] = None, file_name: Optional[str] = None, **kwargs) -> "MessageId":
        return await self.reply_file(photo, caption, file_name, "Image", **kwargs)

    async def reply_video(self, video: Union[str, bytes], caption: Optional[str] = None, file_name: Optional[str] = None, **kwargs) -> "MessageId":
        return await self.reply_file(video, caption, file_name, "Video", **kwargs)

    async def reply_gif(self, gif: Union[str, bytes], caption: Optional[str] = None, file_name: Optional[str] = None, **kwargs) -> "MessageId":
        return await self.reply_file(gif, caption, file_name, "Gif", **kwargs)

    async def reply_music(self, music: Union[str, bytes], caption: Optional[str] = None, file_name: Optional[str] = None, **kwargs) -> "MessageId":
        return await self.reply_file(music, caption, file_name, "Music", **kwargs)

    async def reply_voice(self, voice: Union[str, bytes], caption: Optional[str] = None, file_name: Optional[str] = None, **kwargs) -> "MessageId":
        return await self.reply_file(voice, caption, file_name, "Voice", **kwargs)


class InlineMessage(DataManager):
    client: Optional[Any] = None
    sender_id: Optional[str] = None
    text: Optional[str] = None
    message_id: Optional[str] = None
    chat_id: Optional[str] = None
    file: Optional[File] = None
    location: Optional[Location] = None
    aux_data: Optional[AuxData] = None

    def __init__(
        self,
        sender_id: Optional[str] = None,
        text: Optional[str] = None,
        message_id: Optional[str] = None,
        chat_id: Optional[str] = None,
        file: Optional[File] = None,
        location: Optional[Location] = None,
        aux_data: Optional[AuxData] = None,
        **kwargs
    ):
        super().__init__(
            sender_id=sender_id,
            text=text,
            message_id=message_id,
            chat_id=chat_id,
            file=file,
            location=location,
            aux_data=aux_data,
            **kwargs
        )


class Updates(DataManager):
    updates: Optional[list[Update]] = None
    next_offset_id: Optional[str] = None

    def __init__(
        self,
        updates: Optional[list[Update]] = None,
        next_offset_id: Optional[str] = None,
        **kwargs
    ):
        super().__init__(
            updates=updates,
            next_offset_id=next_offset_id,
            **kwargs
        )


from rubigram.types import Update, InlineMessage
from typing import Union
import re



class Filter:
    def __init__(self, func):
        self.func = func
    
    async def __call__(self, update: Union[Update, InlineMessage]):
        return await self.func(update)
    
    def __and__(self, other: "Filter"):
        async def filter(update):
            return await self(update) and await other(update)
        return Filter(filter)
    
    def __or__(self, other: "Filter"):
        async def filter(update):
            return await self(update) or await other(update)
        return Filter(filter)



async def TEXT(message: Update):
    if isinstance(message, Update):
        return True if message.new_message and message.new_message.text else False
    return False

async def FILE(message: Update):
    if isinstance(message, Update):
        return True if message.new_message and message.new_message.file else False
    return False

async def LIVE(message: Update):
    if isinstance(message, Update):
        return True if message.new_message and message.new_message.live_location else False
    return False

async def POLL(message: Update):
    if isinstance(message, Update):
        return True if message.new_message and message.new_message.poll else False
    return False

async def CONTACT(message: Update):
    if isinstance(message, Update):
        return True if message.new_message and message.new_message.contact_message else False
    return False

async def STICKER(message: Update):
    if isinstance(message, Update):
        return True if message.new_message and message.new_message.sticker else False
    return False

async def LOCATION(message: Update):
    if isinstance(message, Update):
        return True if message.new_message and message.new_message.location else False
    return False

async def FORWARD(message: Update):
    if isinstance(message, Update):
        return True if message.new_message and message.new_message.forwarded_from else False
    return False

async def EDITED(message: Update):
    if isinstance(message, Update):
        return True if message.updated_message else False
    return False

async def PRIVATE(message: Union[Update, InlineMessage]):
    return True if message.chat_id.startswith("b0") else False
    
async def GROUP(message: Union[Update, InlineMessage]):
    return True if message.chat_id.startswith("g0") else False

async def CHANNEL(message: Union[Update, InlineMessage]):
    return True if message.chat_id.startswith("c0") else False

async def FORWARD_BOT(message: Update):
    if isinstance(message, Update) and message.new_message:
        return True if message.new_message.forwarded_from and message.new_message.forwarded_from.type_from == "Bot" else False

async def FORWARD_USER(message: Update):
    if isinstance(message, Update) and message.new_message:
        return True if message.new_message.forwarded_from and message.new_message.forwarded_from.type_from == "User" else False
    
async def FORWARD_CHANNEL(message: Update):
    if isinstance(message, Update) and message.new_message:
        return True if message.new_message.forwarded_from and message.new_message.forwarded_from.type_from == "Channel" else False


text = Filter(TEXT)
file = Filter(FILE)
live = Filter(LIVE)
poll = Filter(POLL)
contact = Filter(CONTACT)
sticker = Filter(STICKER)
location = Filter(LOCATION)
forward = Filter(FORWARD)
edited = Filter(EDITED)
private = Filter(PRIVATE)
group = Filter(GROUP)
channel = Filter(CHANNEL)
forward_bot = Filter(FORWARD_BOT)
forward_user = Filter(FORWARD_USER)
forward_channel = Filter(FORWARD_CHANNEL)


class state(Filter):
    def __init__(self, state: Union[str, list[str]]):
        self.states = state if isinstance(state, list) else [state]
        super().__init__(self.filter)
        
    async def filter(self, update: Union[Update, InlineMessage]):
        user_state = await update.client.state.get_state(update.chat_id)
        return user_state in self.states
            

class command(Filter):
    def __init__(self, command: Union[str, list[str]], prefix: Union[str, list[str]] = "/", case_sensitive: bool = False):
        self.commands = [c if case_sensitive else c.lower() for c in (command if isinstance(command, list) else [command])]
        self.prefixs = prefix if isinstance(prefix, list) else [prefix]
        self.cmds = [p + c for p in self.prefixs for c in self.commands]
        self.case_sensitive = case_sensitive
        super().__init__(self.filter)

    async def filter(self, update: Update):
        if isinstance(update, Update) and update.type == "NewMessage":
            text = update.new_message.text or ""
            text = text if self.case_sensitive else text.lower()
            return any(text.startswith(cmd) for cmd in self.cmds)
        return False
    

class button(Filter):
    def __init__(self, button_id: Union[str, list[str]], prefix: Union[str, list[str]] = "", case_sensitive: bool = False):
        self.button_ids = [btn_id if case_sensitive else btn_id.lower() for btn_id in (button_id if isinstance(button_id, list) else [button_id])]
        self.prefixs = prefix if isinstance(prefix, list) else [prefix]
        self.btn_ids = [p + b for p in self.prefixs for b in self.button_ids]
        self.case_sensitive = case_sensitive
        super().__init__(self.filter)

    async def filter(self, update: InlineMessage):
        if isinstance(update, InlineMessage):
            text = update.text or ""
            text = text if self.case_sensitive else text.lower()
            return any(text.startswith(btn) for btn in self.btn_ids)
        return False

class regex(Filter):
    def __init__(self, pattern: str):
        self.pattern = pattern
        super().__init__(self.filter)

    async def filter(self, update: Union[Update, InlineMessage]):
        text = ""
        if isinstance(update, Update):
            if update.type == "NewMessage":
                text = getattr(update.new_message, "text", "")
            elif update.type == "UpdatedMessage":
                text = getattr(update.updated_message, "text", "")
        elif isinstance(update, InlineMessage):
            text = getattr(update, "text", "")

        if text:
            return bool(re.search(self.pattern, text))
        return False


class chat(Filter):
    def __init__(self, chat_id: Union[str, list[str]]):
        self.chat_id = chat_id
        super().__init__(self.filter)

    async def filter(self, update: Union[Update, InlineMessage]):
        chat_ids = self.chat_id if isinstance(self.chat_id, list) else [self.chat_id]
        return update.chat_id in chat_ids


from aiohttp import ClientSession, FormData
from typing import Any, Optional, Union
from pathlib import Path
from urllib.parse import urlparse
from random import randint
import aiofiles
import os


class Network:
    def __init__(self, token: str) -> None:
        self.token: str = token
        self.session: Optional[ClientSession] = None
        self.api: str = f"https://botapi.rubika.ir/v3/{token}/"

    async def start(self):
        if not self.session:
            self.session = ClientSession()

    async def stop(self):
        if self.session:
            await self.session.close()
            self.session = None

    async def __aenter__(self):
        await self.start()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.stop()

    async def request(self, method: str, json: dict[str, Any]):
        await self.start()
        async with self.session.post(self.api + method, json=json) as response:
            response.raise_for_status()
            data: dict = await response.json()
            return data.get("data")

    async def getBytes(self, url: str) -> bytes:
        await self.start()
        async with self.session.get(url) as response:
            response.raise_for_status()
            return await response.read()

    async def getName(self, url: str) -> str:
        parser = urlparse(url)
        return os.path.basename(parser.path)

    async def requestUpload(self, upload_url: str, file: Union[str, bytes], name: Optional[str] = None):
        data, filename = None, None
        if isinstance(file, str):
            path = Path(file)

            if path.is_file():
                data, filename = path.read_bytes(), name if name else path.name

            elif file.startswith("http"):
                data, filename = await self.getBytes(file), name if name else await self.getName(file)

            else:
                raise Exception(f"Can't find this file : {file}")

        elif isinstance(file, bytes):
            if name:
                data, filename = file, name
            else:
                raise Exception("choice name for bytes file")

        form = FormData()
        form.add_field("file", data, filename=filename, content_type="application/octet-stream")
        await self.start()
        async with self.session.post(upload_url, data=form) as response:
            response.raise_for_status()
            data: dict = await response.json()
            return data.get("data", {})["file_id"]

    async def requestDownload(self, url: str, filename: Optional[str] = None):
        file, name = await self.getBytes(url), filename if filename else await self.getName(url)
        async with aiofiles.open(name, "wb") as f:
            await f.write(file)
            return name